/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "wrappers.h"
#include "list.h"

/*
** Public methods
*/

s_linked_list *list_new(void) { return NULL; }

void list_append(s_linked_list **list, void *content, int size)
{
	s_linked_list	*aux;

	// alloc new element
	aux = (s_linked_list *)do_malloc(sizeof(s_linked_list));

	// fill new struct
	aux->content = (size)?do_memdup(content,size):content;
	aux->next = *list;

	// link new element
	*list = aux;
}

void list_clean(s_linked_list **list, list_callback free_callback)
{
	if(!*list) return;

	list_clean(&((*list)->next),free_callback);

	// free memory
	if(free_callback) free_callback((*list)->content);
	free((*list)->content);
	free((*list));
}

int list_count(s_linked_list **list)
{
	s_linked_list		*aux = *list;
	int			count = 0;

	while(aux)
	{
		count++;
		aux = aux->next;
	}

	return count;
}
