/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>

#include "common.h"
#include "socks4.h"
#include "libnet.h"
#include "debug.h"

/*
** Public methods
*/

void socks4_connect(s_libnet *slib)
{
	unsigned char		*buffer;
	int			read_count;
	s_socks4_request	s_request;

	// fill request structure
	s_request.ver		= SOCKS4_VERSION;			// SOCKS version 4
	s_request.cmd		= SOCKS4_CMD_TCP_CONNECT;		// TCP connect
	s_request.port		= htons(slib->port);			// Port
	s_request.userid 	= 0x001337;				// User ID
	s_request.ip		= slib->hp->h_address.s_addr;		// Ip

	// send request
	net_write(slib,(void *)&s_request,sizeof(s_socks4_request));

	debug(3,"socks4_connect: connection request sent, waiting for reply ...");
	
	// check response
	while(!(read_count = net_read(slib,(void **)&buffer,8)));
	if(read_count == -1) abort("socks4_connect: %s",strerror(errno));

	debug(3,"socks4_connect: response received - r[0]=%u , r[1]=%u",buffer[0],buffer[1]);
	
	if(!buffer[0])
		switch(buffer[1])
		{
			case SOCKS4_RESPONSE_GRANTED: 

				debug(3,"socks4_connect: connection established");

				free(buffer); 
				return;
			break;
			case SOCKS4_RESPONSE_REJECTED: abort("socks4_connect: request rejected");
			break;
			case SOCKS4_RESPONSE_IDENT_FAILED: abort("socks4_connect: ident failed");
			break;
			case SOCKS4_RESPONSE_IDENT_CONFLICT: abort("socks4_connect: ident conflict");
			break;
		}

	abort("socks4_connect: unexpected reply");
}
