/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "common.h"
#include "wrappers.h"

/*=[ Public methods ]=*/

FILE *do_fopen(const char *path, const char *mode)
{
	FILE	*fptr_tmp;

	if(!(fptr_tmp = fopen(path,mode))) abort("fopen: %s (%s)",strerror(errno),path);
	return fptr_tmp;
}

size_t do_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	if((nmemb > 0) && (fwrite(ptr,size,nmemb,stream) != nmemb)) abort("fwrite: %s",strerror(errno));
	return nmemb;	
}

int do_fseek(FILE *stream, long offset, int whence)
{
	if(fseek(stream,offset,whence) == -1) abort("fseek: %s",strerror(errno));
	return 0;
}

long do_ftell(FILE *stream)
{
	long	offset;

	if((offset = ftell(stream)) == -1) abort("ftell: %s",strerror(errno));
	return offset;
}

size_t do_fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	size_t	res;

	res = fread(ptr,size,nmemb,stream);
	if(ferror(stream)) abort("fread: %s",strerror(errno));

	return res;
}

char *do_fgets(char *s, int size, FILE *stream)
{
	char	*res;

	if(!(res = fgets(s,size,stream)) && (ferror(stream)))
		abort("fgets: %s",strerror(errno));

	return res;
}

void *do_mmap(void *start, size_t length, int prot, int flags, int fd, off_t offset)
{
	void	*res;

	if((res = mmap(start,length,prot,flags,fd,offset)) == MAP_FAILED) abort("mmap: %s",strerror(errno));
	return res;
}

int do_munmap(void *start, size_t length)
{
	if(munmap(start,length) == -1) abort("munmap: %s",strerror(errno));
	return 0;
}

void *do_malloc(size_t size)
{
	void	*pointer;

	if(!(pointer = malloc(size))) abort("malloc: %s",strerror(errno));
	return pointer;
}

void *do_calloc(size_t nmemb, size_t size)
{
	void	*pointer;

	if(!(pointer = calloc(nmemb,size))) abort("calloc: %s",strerror(errno));
	return pointer;
}

void *do_realloc(void *ptr, int size)
{
	ptr = (void *)realloc(ptr,size);
	if(!ptr) abort("realloc: %s",strerror(errno));

	return ptr;
}

char *do_strdup(const char *s)
{
	char	*res;

	if(!s) return NULL;
	if(!(res = strdup(s))) abort("strdup: %s",strerror(errno));
	return res;
}

int do_socket(int domain, int type, int protocol)
{
	int	res;

	if((res = socket(domain,type,protocol)) == -1) abort("socket: %s",strerror(errno));
	return res;
}

int do_connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen)
{
	int	res;

	if((res = connect(sockfd,serv_addr,addrlen)) == -1) abort("connect: %s",strerror(errno));
	return res;
}

void *do_memdup(void *ptr, int size)
{
	void	*dest;

	if(!(dest = malloc(size))) abort("memdup: %s",strerror(errno));
	memcpy(dest,ptr,size);

	return dest;
}

DIR *do_opendir(const char *name)
{
	DIR	*res;

	if(!(res = opendir(name))) abort("opendir: %s",strerror(errno));
	return res;
}

int do_stat(const char *file_name, struct stat *buf)
{
	if(stat(file_name,buf)) abort("stat: %s (%s)",strerror(errno),file_name);
	return 0;
}


