/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include <netdb.h>

#include <libhuk/common.h>
#include <libhuk/wrappers.h>
#include <libhuk/libnet.h>
#include <libhuk/list.h>

#include "dispatcher.h"
#include "wlist.h"
#include "brute.h"
#include "session.h"
#include "modules.h"

/*=[ Prototypes ]=*/
void usage(char *);

/*=[ Global variables ]=*/
char		*progname;				// Program name
s_session	*session;				// session object

int main(int ac, char **av)
{
	// identify program name
	progname = strrchr(av[0],'/');
	if(!progname) progname = av[0];
	else { progname++; av[0] = progname; }

	// Print banner
	OUTPUT("-=[ Huk v0.1, by Carlos Barros <barros [at] barrossecurity.com> ]=-\n");
	OUTPUT("-=[ http://www.barrossecurity.com/  -  http://www.gotfault.net/ ]=-\n\n");

	// build modules list
	modules_get_modules();

	// create session object
	session = session_new(ac,av);
	if(session->error) usage(session->error_message);

	// save session before starting
	session_save(session);

	// start brute force attack
	OUTPUT("Relax while I'm brute forcing. Press CTRL+\\ to abort or CTRL+C for status\n");
	if(do_brute_force() == ERROR)
		OUTPUT_ERROR("Aborted by the user\n");

	// save session after finishing
	session_save(session);

	// clean up things
	modules_free();
	session_free(session);

	return EXIT_SUCCESS;
}

void usage(char *error)
{
	if(error) OUTPUT_ERROR("%s: %s\n",progname,error);
	OUTPUT_ERROR("Usage: %s <-m module> <-h host> [options]\n",progname);
	OUTPUT_ERROR("Options:\n");
	OUTPUT_ERROR("\t-n name\t\t- Session name (default: noname)\n");
	OUTPUT_ERROR("\t-R\t\t- Restore session\n");
	OUTPUT_ERROR("\t-p port\t\t- Target port (default: module dependant)\n");
	OUTPUT_ERROR("\t-u username\t- Single username (default: %s)\n",DEFAULT_USERNAME);
	OUTPUT_ERROR("\t-U list\t\t- Username list file\n");
	OUTPUT_ERROR("\t-w password\t- Single password\n");
	OUTPUT_ERROR("\t-W list\t\t- Password list file (default: %s)\n",DEFAULT_WORDLIST);
	OUTPUT_ERROR("\t-C list\t\t- Combo list file (username:password)\n");
	OUTPUT_ERROR("\t-c count\t- Max simultaneous connection (default: %d, max: %d)\n",DEFAULT_CONNECTIONS,MAX_CONNECTIONS);
	OUTPUT_ERROR("\t-t timeout\t- Connection timeout (default: %ds)\n",DEFAULT_TIMEOUT);
	OUTPUT_ERROR("\t-x param\t- Module dependent parameter (name=value)\n\t\t\t- Use -x %s to display module specific help\n",MODULE_USAGE_TRIGGER);
	OUTPUT_ERROR("\t-v\t\t- Increase verbose level\n");
	OUTPUT_ERROR("Modules:\n");

	modules_list();
	
	exit(EXIT_FAILURE);
}

