/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <libhuk/common.h>
#include <libhuk/libnet.h>
#include <libhuk/wrappers.h>
#include <libhuk/debug.h>
#include <libhuk/modules.h>

#include "mod_fastiron.h"

/*=[ Module definition ]=*/
#define MODULE_KEY		"fastiron"
#define MODULE_DESCRIPTION	"FastIronII switch brute force"
#define MODULE_PORT		23

DEFINE_MODULE(
		MODULE_KEY,				// Module key
		MODULE_DESCRIPTION,			// Module description
		MODULE_PORT,				// Module default port
		NULL,					// Module params
		NULL,					// Module validate params routine
		fastiron_module_start			// Module entry point
);

/*
** Public methods
*/

int fastiron_module_start(s_mod_context *ctx)
{
	char		*buffer;
	int		read_count;

	// read the login prompt
	while(!(read_count = net_read(ctx->libnet,(void **)&buffer,0)));
	if(read_count == -1) return BRUTE_ERROR;

	debug(1,"fastiron_module_start: banner received");
	debug(2,"fastiron_module_start: banner = %s",buffer);

	// verify login prompt;
	if(!strstr(buffer,LOGIN_PROMPT)) 
	{ 
		debug(1,"fastiron_module_start: invalid banner");
		debug(2,"fastiron_module_start: received banner = %s",buffer);
		
		free(buffer); 
		return BRUTE_ERROR; 
	}
	free(buffer);

	debug(1,"fastiron_module_start: banner check succeed");
	
	// prepare password
	buffer = do_malloc(strlen(ctx->passwd) + 3);
	sprintf(buffer,"%s\r\n",ctx->passwd);

	// send password
	if(net_write(ctx->libnet,(void *)buffer,strlen(buffer)) == ERROR)
	{	
		free(buffer);
		return BRUTE_ERROR;
	}
	free(buffer);

	debug(1,"fastiron_module_start: authentication sent, waiting for reply ...");

	// read response
	while(!(read_count = net_read(ctx->libnet,(void **)&buffer,0)));
	if(read_count == -1) return BRUTE_ERROR;

	debug(1,"fastiron_module_start: response received");
	debug(2,"fastiron_module_start: response = %s",buffer);
	
	// check response
	if(!strstr(buffer,FAILURE_MSG)) { free(buffer); return VALID_LOGIN; }
	free(buffer);

	return INVALID_LOGIN;
}

