/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <libhuk/common.h>
#include <libhuk/libnet.h>
#include <libhuk/wrappers.h>
#include <libhuk/base64.h>
#include <libhuk/debug.h>
#include <libhuk/modules.h>

#include "mod_http_basic.h"

/*=[ Module definition ]=*/
#define MODULE_KEY		"http_basic"
#define MODULE_DESCRIPTION	"HTTP Basic authentication brute force"
#define MODULE_PORT		80

// module parameters
s_module_params_list		http_basic_params[] =
{
	{"host","traget host name",1},
	{"path","target path",1},
	END_OF_PARAMS_LIST
};

DEFINE_MODULE(
		MODULE_KEY,				// Module key
		MODULE_DESCRIPTION,			// Module description
		MODULE_PORT,				// Module default port
		http_basic_params,			// Module params
		NULL,					// Module validate params routine
		http_basic_module_start			// Module entry point
);

/*
** Public methods
*/

int http_basic_module_start(s_mod_context *ctx)
{
	char		*host, *path, *challange, *buffer, *code;
	int		read_count;

	// retrieve params
	host = modules_get_param(&ctx->params,"host");
	path = modules_get_param(&ctx->params,"path");

	// build the challange
	buffer = (char *)do_malloc(strlen(ctx->user)+strlen(ctx->passwd)+2);
	sprintf(buffer,"%s:%s",ctx->user,ctx->passwd);
	challange = base64_encode(buffer,strlen(buffer));
	free(buffer);
	
	// build the request buffer
	buffer = (char *)do_malloc(strlen(path)+strlen(host)+strlen(challange)+60); // just to be safe =)
	sprintf(buffer,"GET %s%s HTTP/1.0\r\nHost: %s\r\nAuthorization: Basic %s\r\n\r\n",((*path == '/')?"":"/"),path,host,challange);

	// send request
	if(net_write(ctx->libnet,(void *)buffer,strlen(buffer)) == ERROR)
	{
		free(buffer);
		return BRUTE_ERROR;
	}
	free(buffer);

	debug(1,"http_basic_module_start: request sent, waiting for reply ...");
	
	// get response
	while(!(read_count = net_read(ctx->libnet,(void **)&buffer,12)));
	if(read_count == -1) return BRUTE_ERROR;

	buffer[read_count] = 0;

	debug(1,"http_basic_module_start: response received");
	debug(2,"http_basic_module_start: response = %s",buffer);
	
	// parse response
	if(!(code = strstr(buffer,"HTTP/1."))) 
	{
		debug(1,"http_basic_module_start: unexpected response");
		return BRUTE_ERROR;
	}
	else code += strlen("HTTP/1.x");

	// locate return code
	while(*code != ' ') code++; while(*code == ' ') code++;

	// return code starting with something other than 4 indicates success
	if((*code != '4') || (*(code+1) != '0') || (*(code+2) != '1')) return VALID_LOGIN;

	debug(1,"http_basic_module_start: rejected with code %c%c%c",*code,*(code+1),*(code+2));
	
	return INVALID_LOGIN;
}

