/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <libssh/libssh.h>

#include <libhuk/common.h>
#include <libhuk/libnet.h>
#include <libhuk/wrappers.h>
#include <libhuk/debug.h>
#include <libhuk/modules.h>

#include "mod_ssh.h"

/*=[ Module definition ]=*/
#define MODULE_KEY		"ssh"
#define MODULE_DESCRIPTION	"SSH brute force"
#define MODULE_PORT		22

DEFINE_MODULE(
		MODULE_KEY,				// Module key
		MODULE_DESCRIPTION,			// Module description
		MODULE_PORT,				// Module default port
		NULL,					// Module params
		NULL,					// Module validate params routine
		ssh_module_start			// Module entry point
);

/*
** Public methods
*/

int ssh_module_start(s_mod_context *ctx)
{
	SSH_OPTIONS		*ssh_options;
	SSH_SESSION		*ssh_session;
	int			ssh_auth_result,
				ssh_try_count;

	// setup options 
	ssh_options = options_new();

	// no compression
	options_set_wanted_method(ssh_options,KEX_COMP_C_S,"none");
	options_set_wanted_method(ssh_options,KEX_COMP_S_C,"none");
	options_set_username(ssh_options,ctx->user);
	options_set_fd(ssh_options,ctx->libnet->sock);

	// open ssh session
	// actually we do not "connect" as we are already
	// connected. See options_set_fd above.
	ssh_session = ssh_connect(ssh_options);
	if(!ssh_session)
	{
		debug(1,"ssh_module_start: %s",ssh_get_error(ssh_session));
		return BRUTE_ERROR;
	}

	// try auth using kbd-interactive method
	ssh_try_count = 0;
	while(((ssh_auth_result = ssh_userauth_kbdint(ssh_session,ctx->user,NULL)) == SSH_AUTH_INFO) && (ssh_try_count < SSH_MAX_TRY))
	{
		ssh_userauth_kbdint_setanswer(ssh_session,ssh_try_count,ctx->passwd);
		ssh_try_count++;
	}

	// if fails, try password method
	if(ssh_auth_result != SSH_AUTH_SUCCESS)
	{
		debug(1,"ssh_module_start: kbd-interactive failed, trying password method");
		ssh_auth_result = ssh_userauth_password(ssh_session,NULL,ctx->passwd);
	}

	ssh_disconnect(ssh_session);	// this will free ssh_options

	return (ssh_auth_result == SSH_AUTH_SUCCESS)?VALID_LOGIN:INVALID_LOGIN;
}

