/*
 * Huk: a multi-threaded modular bruteforce tool 
 *
 * Written by Carlos Barros <barros [at] barrossecurity [dot] com>
 *
 * Copyright (C) 2006, Barrossecurity
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __SESSION_H__
#define	__SESSION_H__

#include "modules.h"

/*=[ Macros ]=*/
#define	session_abort(msg)		{ session.error = 1; session.error_message = msg; return &session; }
#define	p_session_abort(msg)		{ session->error = 1; session->error_message = msg; return; }

#define	SESSION_FILE_GET_STR(i,o)	((session_header->i)?session_contents+(session_header->i)+o:NULL)
#define	SESSION_FILE_GET_INT(i)		session_header->i

/*=[ Defines ]=*/
#define	DEFAULT_SESSION_NAME	"noname"
#define MAX_SESSION_NAME	10

/*=[ Structs ]=*/
typedef struct
{
	char			*name;				// session name
	s_module_definition	*module;                        // module to use
	s_linked_list		*module_params;                 // module dependent params
	s_libnet		*libnet;                        // network interface
	s_wlist			*wlist;                         // wordlist descriptor
	int			max_conn;                       // max connections
	int			conn_timeout;                   // connection timeout
	s_linked_list		*valid_logins;                  // list of valid logins found
	int			tried_logins;                   // tried logins count
	time_t			elapsed_time;			// accumulated time
	time_t			start_time;                     // brute force start time
	int			abort_bruteforce;        	// abort flag

	int			error;				// error flag
	char			*error_message;			// error message
}s_session;

typedef struct
{

	char			*passwd_list;
	char			*single_passwd;
	char			*user_list;
	char			*single_user;
	char			*combo_list;
	char			*host;
	int			port;
	int			restore_flag;
	int			m_usage;
}s_session_options;

typedef struct
{
	char			*username;
	char			*password;
}s_login_pair;

/*
** This struct is just a list of indexes pointing
** into the string table, that goes after it
*/
typedef struct
{
	int			module_key;
	int			module_params;
	int			module_params_count;
	int			host;
	int			port;
	int			max_conn;
	int			conn_timeout;
	int			valid_logins;
	int			valid_logins_count;
	int			tried_logins;
	time_t			elapsed_time;
	int			user_list;
	int			passwd_list;
	int			combo_list;
	int			single_user;
	int			single_passwd;
}s_session_file;

/*=[ Prototypes ]=*/
s_session *session_new(int , char **);
void session_free(s_session *);
void session_new_valid_login(s_session *, char *, char *);
void session_task_finished(s_session *);
int session_next_pair(s_session *);
char *session_current_user(s_session *);
char *session_current_pass(s_session *);
void session_show_status(s_session *);
void session_save(s_session *);
void session_restore(s_session *, s_session_options *);

#endif

